<?php

namespace Magento\FirstDataGateway\Helper;

use \Magento\Framework\App\Helper\AbstractHelper;

class Request extends AbstractHelper
{
	private $debug = false;
	protected $logger;
	protected $_transportBuilder;

	public function __construct(
		\Magento\Framework\App\Helper\Context $context,
		\Magento\Framework\Mail\Template\TransportBuilder $transportBuilder
	)
	{
		$this->logger = \Magento\Framework\App\ObjectManager::getInstance()->get(\Psr\Log\LoggerInterface::class);
		$this->_transportBuilder = $transportBuilder;
		return parent::__construct($context);
	}

	public function testRedir($sharedKey) {
		$objectManager = \Magento\Framework\App\ObjectManager::getInstance();
		$storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');
		$store = $storeManager->getStore();
		$root = $store->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_WEB);
		if(!isset($_GET['id'])) return $root;
		if(!isset($_GET['sess'])) return $root;
		if(md5($_GET['id'] . $sharedKey) != $_GET['sess']) return $root;
		
		return '';
	}

	public function request($posId, $sharedKey, $notification = false) {
	    $this->_log('FirstData req start', true);
	    if(!$_POST) {
	        $this->_log('no input');
	        return false;
	    }
//	    error_log('request post=' . print_r($_POST,true));
	    $error = false;
	    $acode = explode(':', $_POST['approval_code']);
	    $status = $acode[0];
	    
	    if($notification) {
	        $data = array(
	            'chargetotal' => $_POST['chargetotal'],
	            'currency' => $_POST['currency'],
	            'txndatetime' => $_POST['txndatetime'],
	            'storename' => trim($posId),
	            'approval_code' => $_POST['approval_code']
	        );
	        $hash = $_POST['notification_hash'];
	    }
	    else {
	        $data = array(
	            'approval_code' => $_POST['approval_code'],
	            'chargetotal' => $_POST['chargetotal'],
	            'currency' => $_POST['currency'],
	            'txndatetime' => $_POST['txndatetime'],
	            'storename' => trim($posId)
	        );
	        $hash = $_POST['response_hash'];
	    }
	    $chash = $this->createExtendedHash($data, $sharedKey);
	    if($hash != $chash) {
	        $text = "Wrong controlData in confirmation request.\r\n  order id: ".$_POST['oid'];
	        if(isset($data['ipgTransactionId']))
	            $text .= "\r\n  Transadtion id: ".$_POST['ipgTransactionId'];
	            $this->_log($text);
	            
	            return false;
	    }
	    
	    $data['chargetotal'] = 1 * str_replace(',', '.', $data['chargetotal']);
	    
	    $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
	    $order = $objectManager->create('Magento\Sales\Model\Order')->loadByIncrementId($_POST['oid']);
	    if($order->getId() == 0)  {
	        $text = "Invalid order id in confirmation request.\r\n  order id: ".$_POST['oid'];
	        $this->_log($text);
	        return false;
	    }
	    
	    if($status == 'N') {
	        $text = "Payment error.\r\n  error code: " . $error . "\r\n  msg: " . $acode[2] . "\r\n  order id: " . $_POST['oid'];
	        if(isset($_POST['ipgTransactionId']))
	            $text .= "\r\n  Transadtion id: ".$_POST['ipgTransactionId'];
	            $this->_log($text);
	            $this->orderStatus($order, $status, $data);
	            
	            return false;
	    }
	    
	    if(!$this->orderStatus($order, $status, $data)) return false;
	    $order->save();

	    $text = "Transaction complete\r\n  Transadtion id: ". $_POST['ipgTransactionId'];
	    $this->_log($text);
	    
	    return true;
	}

	public function orderStatus($order, $status, $data) {
	    $ret = true;
	    $opis = isset(self::$statusy[$status]) ? self::$statusy[$status] : '???';
	    $payment = $order->getPayment();
	    $order->addStatusHistoryComment('FirstData status: [' . $status . '] ' . $opis . '');
	    $payment->setData('firstdata_status', $status);
	    
	    $transaction_id = (isset($_POST['ipgTransactionId']) ? $_POST['ipgTransactionId'] : '');
	    
	    $msg = array(
	        'status' => $status,
	        'txndate_processed' => $_POST['txndate_processed'],
	        'approval_code' => $_POST['approval_code'],
	        'chargetotal' => $_POST['chargetotal'],
	        'currency' => $_POST['currency'],
	    );
	    
	    if($status == 'Y') {
	        # Kwota
	        if(round(floatval($order->getGrandTotal()) * 100) != $data['chargetotal']) {
	            $state = \Magento\Sales\Model\Order::STATE_HOLDED;
	            
	            $text = "Wrong payment amount.\r\n  order id: ".$order->getIncrementId();
	            $text .= "\r\n  FD tid: " . $transaction_id . ', ' . round(floatval($order->getGrandTotal()) * 100) . ' expected, got ' . $data['chargetotal'];
                $this->_log($text);
	        }
	        else $state = \Magento\Sales\Model\Order::STATE_PROCESSING;
	        
	        if($payment->getAmountPaid() == 0) {
	            if($order->getStatus() != $state) $order->setStatus($state);
	            $payment->setTransactionId($transaction_id);
	            $payment->setIsTransactionClosed(0);
	            $payment->setCurrencyCode('PLN');
	            $payment->setPreparedMessage('FirstData status ' . $status);
	            $payment->registerCaptureNotification($data['chargetotal']);
	            $order->save();
	        }
	        $msg['ipgTransactionId'] = $_POST['ipgTransactionId'];
	    }
	    elseif($status == '?') {
	        $state = \Magento\Sales\Model\Order::STATE_PENDING_PAYMENT;
	        if($order->getStatus() != $state) $order->setStatus($state);
	        
	        $msg['ipgTransactionId'] = $transaction_id;
	    }
	    else {
	        $state = \Magento\Sales\Model\Order::STATE_HOLDED;
	        if($order->getStatus() != $state) $order->setStatus($state);
	        
	        $ret = false;
	        $msg['fail_reason'] = $_POST['fail_reason'];
	    }
	    return $ret;
	}

	private function createExtendedHash($data, $sharedKey) {
	    $stringToHash = join('|', $data);
	    //	    error_log('s=' . $stringToHash . ' k=' . $this->sharedKey);
	    $hash = hash_hmac('sha256', $stringToHash, $sharedKey, true);
	    
	    return base64_encode($hash);
	}
	
	private function _log($text, $info = false)
	{
		if($info) $this->logger->info('FirstData ' . $text);
		else $this->logger->error('FirstData ' . $text);
	}
	
}

?>
